package usingjsp;

import java.io.*;

// A class that performs file locking by renaming

public class FileLock
{
    protected File mainFile;
    protected File lockedFile;
    protected File tempFile;
    protected boolean rewroteNew;
    protected boolean locked;

/** Create a lock for a named file */
    public FileLock(String filename)
    {
        mainFile = new File(filename);
        lockedFile = new File(filename+".lck");
        rewroteNew = false;
        locked = false;
    }

    public boolean lock()
        throws java.io.IOException
    {
        return lock(30, 1000);
    }

    public boolean lock(int maxRetries, int waitTimeMillis)
        throws java.io.IOException
    {
        boolean renameSucceeded = false;

// If the file is already locked, no need to go through the locking
// procedure again
        if (locked) return true;

// Loop until the file is available or there have been too many tries
        for (int i=0; i < maxRetries; i++)
        {

// Make sure the original file exists
            if (mainFile.exists())
            {
               if (mainFile.renameTo(lockedFile))
               {

// If the rename succeeds, the file is locked
                   renameSucceeded = true;
                   break;
               }
            }
            else
            {
// If the original file doesn't exist, try creating it. If the create fails,
// someone must have the file locked already
                if (lockedFile.createNewFile())
                {
                    renameSucceeded = true;
                    break;
                }
            }

// If at this point the rename isn't succeeded, wait for a specified
// period and try again
            try
            {
                Thread.sleep(waitTimeMillis);
            }
            catch (Exception ignore)
            {
            }
        }

// If the rename failed, return false to indicate that the file
// has been locked
        if (!renameSucceeded)
        {
            return false;
        }

        locked = true;
        rewroteNew = false;

        return true;
    }

/** Unlock the locked file */
    public void unlock()
        throws IOException
    {
// If the file hasn't been locked, no need to unlock
        if (!locked) return;

// If the file was rewritten to a temp file, rename the temp file
// back to the original and delete the lock file
        if (rewroteNew)
        {
            lockedFile.delete();
            tempFile.renameTo(mainFile);
        }
        else
        {
// If there was no rewrite, just rename the lock file back to the original
            lockedFile.renameTo(mainFile);
        }

        locked = false;
        rewroteNew = false;
    }

/** Get an input stream to read the locked file */
    public InputStream getInputStream()
        throws IOException
    {
// Only allow reading if the file has been locked
        if (!locked)
        {
            throw new IOException("File not locked");
        }
// Create an input stream to read the locked file
        return new FileInputStream(lockedFile);
    }

/** Get a reader to read the locked file */
    public Reader getReader()
        throws IOException
    {
// Only allow reading if the file has been locked
        if (!locked)
        {
            throw new IOException("File not locked");
        }
// Create a reader to read the locked file
        return new FileReader(lockedFile);
    }

/** Get an output stream to write the locked file */
    public OutputStream getOutputStream()
        throws IOException
    {
// By default, rewrite the file instead of append (matches the
// convention for the FileOutputStream constructor)
        return getOutputStream(false);
    }

/** Get an output stream to write or append to the locked file */
    public OutputStream getOutputStream(boolean appendFile)
        throws IOException
    {
// Only allow writing if the file has been locked
        if (!locked)
        {
            throw new IOException("File not locked");
        }
// Create an output stream for writing the file
        return new FileOutputStream(lockedFile.getAbsolutePath(),
            appendFile);
    }

/** Get a writer to write the locked file */
    public Writer getWriter()
        throws IOException
    {
// By default, rewrite the file instead of append (matches the
// convention for the FileOutputStream constructor)
        return getWriter(false);
    }

/** Get a writer to write or append to the locked file */
    public Writer getWriter(boolean appendFile)
        throws IOException
    {
// Only allow writing if the file has been locked
        if (!locked)
        {
            throw new IOException("File not locked");
        }
// Create an output stream for writing the file
        return new FileWriter(lockedFile.getAbsolutePath(),
            appendFile);
    }

/** Get a stream to create a new copy of the file */
    public OutputStream getRewriteOutputStream()
        throws IOException
    {
// Only allow writing if the file has been locked
        if (!locked)
        {
            throw new IOException("File not locked");
        }

// Create a new file for creating a new copy of the locked file
        tempFile = new File(mainFile.getAbsolutePath()+".tmp");
        rewroteNew = true;

// Create an output stream for writing the new copy
        return new FileOutputStream(tempFile);
    }

/** Get a writer to create a new copy of the file */
    public Writer getRewriteWriter()
        throws IOException
    {
// Only allow writing if the file has been locked
        if (!locked)
        {
            throw new IOException("File not locked");
        }

// Create a new file for creating a new copy of the locked file
        tempFile = new File(mainFile.getAbsolutePath()+".tmp");
        rewroteNew = true;

// Create an output stream for writing the new copy
        return new FileWriter(tempFile);
    }

/** Get a RandomAccessFile object for updating the file */
    public RandomAccessFile getRandomAccessFile()
        throws IOException
    {
// Only allow writing if the file has been locked
        if (!locked)
        {
            throw new IOException("File not locked");
        }

// Create a new RandomAccessFile object for updating the locked file
        return new RandomAccessFile(lockedFile, "rw");
    }
}

