package usingjsp;

import java.sql.*;
import java.util.*;

/** A base class for objects that map to database tables */

public abstract class DatabaseObject
{
/** Gets all the objects in a table */
    public Vector getAll(Connection conn)
        throws SQLException
    {
        return getAll(conn, null, null);
    }

/** Gets all the objects in a table matching a specific where clause */
    public Vector getAll(Connection conn, String whereClause)
        throws SQLException
    {
        return getAll(conn, whereClause, null);
    }

/** Gets all the objects in a table matching a specific where clause 
    and allows you to specify additional tables that are used in the
    where clause. */
    public Vector getAll(Connection conn, String whereClause,
        String additionalTables)
        throws SQLException
    {
        Statement s = null;

        try
        {
            s = conn.createStatement();

            Vector v = new Vector();

// Build the query. The basic query is "select <fields> from table"
            String query = "select "+getFieldList()+" from "+getTableName();

// Add the additional tables if needed
            if (additionalTables != null)
            {
                query = query + "," + additionalTables;
            }

// Add the where clause if needed
            if (whereClause != null)
            {
                query = query + " where "+whereClause;
            }

// Perform the query
            ResultSet results = s.executeQuery(query);

// Create a vector of the results
            while (results.next())
            {
                v.addElement(createInstance(results));
            }

            return v;
        }
        finally
        {
            if (s != null)
            {
                try { s.close(); } catch (Exception ignore) {}
            }
        }
    }

/** Executes an arbitrary query string that should return all the
 *  fields in the table just like the other queries. */
    public Vector executeQuery(Connection conn, String query)
        throws SQLException
    {
        Statement s = null;

        try
        {
            s = conn.createStatement();

            Vector v = new Vector();

// Execute the query
            ResultSet results = s.executeQuery(query);

// Create a vector containing the results
            while (results.next())
            {
                v.addElement(createInstance(results));
            }

            return v;
        }
        finally
        {
            if (s != null)
            {
                try { s.close(); } catch (Exception ignore) {}
            }
        }
    }

/** Inserts an object into the database */
    public int insert(Connection conn)
        throws SQLException
    {
        PreparedStatement s = null;

        try
        {
// Ask this object to create its own insert statement
            s = conn.prepareStatement(getInsertStatement());

// Populate the insert statement with the data values
            prepareInsertStatement(s);

// Perform the insert
            return s.executeUpdate();
        }
        finally
        {
            if (s != null)
            {
                try { s.close(); } catch (Exception ignore) {}
            }
        }
    }

// Returns a list of the fields in the table (used for select)
    public abstract String getFieldList();

/** Returns a list of fields in the table while specifying a specific
 *  table for the field names, aliasing the table.field names back to the
 *  original name. For example, you might want to query from table A and
 *  table B, each containing a field F. If you do "select F from A,B ..." you
 *  might get an error because F is ambiguous. You need to do
 *  "select A.F as F from A,B ...". This routine takes the original field
 *  list and creates a list of such aliases.
 */
    public String getFieldList(String tableName)
    {
// Create a tokenizer to parse through the original list
        StringTokenizer fieldList = new StringTokenizer(
            getFieldList(), ",");

// Create the string buffer to hold the resulting list
        StringBuffer newList = new StringBuffer();
        boolean first = true;

        while (fieldList.hasMoreTokens())
        {
            String field = fieldList.nextToken();

            if (!first) newList.append(',');
            first = false;
            newList.append(tableName);
            newList.append('.');
            newList.append(field);
            newList.append(" as ");
            newList.append(field);
        }

        return newList.toString();
    }

/** Creates an insert statement to be used in a PreparedStatement */
    public abstract String getInsertStatement();

/** Stores this object's data in the PreparedStatement returned by
 *  prepareInsertStatement. */
    public abstract void prepareInsertStatement(PreparedStatement s)
        throws SQLException;

/** Updates the database row containing this object */
    public int update(Connection conn)
        throws SQLException
    {
        PreparedStatement s = null;

        try
        {
            s = conn.prepareStatement(getUpdateStatement());

            prepareUpdateStatement(s);

            return s.executeUpdate();
        }
        finally
        {
            if (s != null)
            {
                try { s.close(); } catch (Exception ignore) {}
            }
        }
    }

    public abstract String getUpdateStatement();
    public abstract void prepareUpdateStatement(PreparedStatement s)
        throws SQLException;

/** Deletes this object from the database */
    public int delete(Connection conn)
        throws SQLException
    {
        PreparedStatement s = null;

        try
        {
            s = conn.prepareStatement(getDeleteStatement());

            prepareDeleteStatement(s);

            return s.executeUpdate();
        }
        finally
        {
            if (s != null)
            {
                try { s.close(); } catch (Exception ignore) {}
            }
        }
    }

/** Returns the sequence number used to insert this object. This method
 *  is very database-dependent. This version is geared towards the
 *  mySQL database.
 */
    public int getSequenceNumber(Connection conn)
        throws SQLException
    {
        Statement s = null;

        try
        {
            s = conn.createStatement();

            ResultSet results = s.executeQuery(
                "select last_insert_id()");

            if (results.next())
            {
                return results.getInt(1);
            }
            else
            {
                throw new SQLException(
                    "Unable to generate sequence number");
            }

        }
        finally
        {
            if (s != null)
            {
                try { s.close(); } catch (Exception ignore) {}
            }
        }
    }

    public abstract String getDeleteStatement();
    public abstract void prepareDeleteStatement(PreparedStatement s)
        throws SQLException;

    public abstract String getTableName();
    public abstract DatabaseObject createInstance(ResultSet results)
        throws SQLException;

    public String getSequenceGenerator() { return null; }
}
